'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var preact = require('preact');
require('reshape-preact-components');
var EventEmitter = _interopDefault(require('events'));
var inEU = _interopDefault(require('@segment/in-eu'));
var fetch = _interopDefault(require('unfetch'));
var assign = _interopDefault(require('object-assign'));
var Toggle = _interopDefault(require('@hashicorp/hashi-toggle'));
var Button = _interopDefault(require('@hashicorp/hashi-button'));
var cookies = _interopDefault(require('js-cookie'));

function ConsentBanner(props) {
  return preact.h(
    'div',
    { 'class': 'g-consent-manager-banner flex-centered-row' },
    preact.h(
      'span',
      null,
      'We use cookies & other similar technology to collect data to improve your experience on our site, as described in our',
      ' ',
      preact.h(
        'a',
        {
          href: props.privacyPolicyLink,
          target: '_blank',
          rel: 'noopener noreferrer'
        },
        'Privacy Policy'
      ),
      '.'
    ),
    preact.h(
      'div',
      null,
      preact.h(
        'a',
        { className: 'manage-link', onClick: props.onManagePreferences },
        'Manage Preferences'
      ),
      preact.h(
        'button',
        { onClick: props.onAccept },
        'Accept'
      )
    )
  );
}

/*
  integrations.js
  Fetches integrations from Segment given write key and applies user-defined
  overrides. Combines with custom scripts to give a complete view into
  scripts that the site is trying to load. Finally, groups it all by category.
*/

function getIntegrations(segmentServices, additionalServices, segmentWriteKey, utilServerRoot) {
  return fetchIntegrationsFromSegment(segmentWriteKey, utilServerRoot).then(zipIntegrations.bind(this, segmentServices, additionalServices)).then(groupByCategory);
}

// Get integrations from Segment by write key
function fetchIntegrationsFromSegment(segmentWriteKey, utilServerRoot) {
  return fetch(utilServerRoot + '/seg_integrations/' + segmentWriteKey).then(function (res) {
    if (!res.ok) {
      throw new Error('Failed to fetch integrations: HTTP ' + res.status + ' ' + res.statusText);
    }

    return res.json();
  });
}

// Include developer overrides for fetched Segment integrations
function zipIntegrations(segmentOverrides, additionalServices, segmentOriginated) {
  // Create Segment integration object with name as key for quick retrieval
  var segmentObj = segmentOriginated.reduce(function (obj, segment) {
    obj[segment.name] = segment;
    obj[segment.name].origin = 'segment';
    return obj;
  }, {});

  // Go through each developer-defined segment service and update
  // segmentObj with any overrides
  for (var i = 0; i < segmentOverrides.length; i++) {
    if (segmentObj.hasOwnProperty(segmentOverrides[i].name)) {
      segmentObj[segmentOverrides[i].name] = assign(segmentObj[segmentOverrides[i].name], segmentOverrides[i]);
    } else {
      /* eslint-disable-next-line */
      console.warn('Unmatched integration config for ' + segmentOverrides[i].name);
    }
  }

  // Include additional (non-Segment) services
  var allIntegrations = additionalServices ? additionalServices.map(function (integration) {
    integration.origin = 'custom';
    return integration;
  }) : [];

  for (var key in segmentObj) {
    allIntegrations.push(segmentObj[key]);
  }

  // Returns array with all integrations, with origin key (segment or custom)
  return allIntegrations;
}

// Group by category for display purposes
function groupByCategory(integrations) {
  var grouped = {};

  integrations.forEach(function (integration) {
    if (grouped.hasOwnProperty(integration.category)) {
      grouped[integration.category].push(integration);
    } else {
      grouped[integration.category] = [integration];
    }
  });

  return grouped;
}

function CloseIcon() {
  return preact.h(
    "svg",
    {
      width: "16",
      height: "16",
      viewBox: "0 0 16 16",
      xmlns: "http://www.w3.org/2000/svg"
    },
    preact.h(
      "title",
      null,
      "Close Button"
    ),
    preact.h("path", {
      d: "M1.778 1.778l12.541 12.541m-.097-12.541L1.681 14.319",
      stroke: "#FFF",
      strokeWidth: "2",
      fill: "none",
      strokeLinecap: "square"
    })
  );
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ConsentPreferences = function (_Component) {
  _inherits(ConsentPreferences, _Component);

  function ConsentPreferences(props) {
    _classCallCheck(this, ConsentPreferences);

    var _this = _possibleConstructorReturn(this, (ConsentPreferences.__proto__ || Object.getPrototypeOf(ConsentPreferences)).call(this, props));

    _this.state = {
      version: props.version || 0,
      showConfirmationDialog: false,
      groupedIntegrations: {}, // Integrations object grouped by category
      consent: props.preferences, // Consent preferences object
      categories: props.categories.reduce(function (obj, category) {
        obj[category.name] = category.description;
        return obj;
      }, {}), // Transform categories description array to object
      showCategories: {} // Category toggle state object
    };

    _this.handleFold = _this.handleFold.bind(_this);
    _this.getCategoryToggle = _this.getCategoryToggle.bind(_this);
    return _this;
  }

  _createClass(ConsentPreferences, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      // Set up integrations
      getIntegrations(this.props.segmentServices, this.props.additionalServices, this.props.segmentWriteKey, this.props.utilServerRoot).then(function (groupedIntegrations) {
        _this2.setState({ groupedIntegrations: groupedIntegrations });
      });
    }

    // Handler for when user toggles a category or individual integration

  }, {
    key: 'handleToggle',
    value: function handleToggle(integrationName, integrationOrigin, newValue) {
      var _this3 = this;

      var currentConsent = this.state.consent || {};

      // If user has { loadAll: true } and they toggle, make sure to preserve
      // the loadAll settings by setting everything to true first. This setting
      // gets reset at the end of the function.
      if (currentConsent.loadAll) {
        Object.keys(this.state.groupedIntegrations).forEach(function (category) {
          _this3.state.groupedIntegrations[category].forEach(function (tool) {
            currentConsent[tool.origin] = currentConsent[tool.origin] || {};
            currentConsent[tool.origin][tool.name] = true;
          });
        });
      }

      if (integrationOrigin === 'categories') {
        // User toggled a category toggle
        // Go through list of groupedIntegrations
        this.state.groupedIntegrations[integrationName].forEach(function (tool) {
          currentConsent[tool.origin] = currentConsent[tool.origin] || {};
          currentConsent[tool.origin][tool.name] = newValue;
        });
      } else {
        // User toggled an individual tool toggle
        currentConsent[integrationOrigin] = currentConsent[integrationOrigin] || {};
        currentConsent[integrationOrigin][integrationName] = newValue;
      }

      delete currentConsent.loadAll;

      this.setState({ consent: currentConsent });
    }

    // Handler for folding categories

  }, {
    key: 'handleFold',
    value: function handleFold(categoryName) {
      var categories = this.state.showCategories;
      categories[categoryName] = !categories[categoryName];
      this.setState({ showCategories: categories });
    }

    // Calculate value of category toggle

  }, {
    key: 'getCategoryToggle',
    value: function getCategoryToggle(category) {
      var _this4 = this;

      return this.state.groupedIntegrations[category].some(function (tool) {
        return _this4.state.consent && _this4.state.consent.hasOwnProperty(tool.origin) && _this4.state.consent[tool.origin].hasOwnProperty(tool.name) && _this4.state.consent[tool.origin][tool.name];
      });
    }

    // Build each individual category and integration row

  }, {
    key: 'buildCategory',
    value: function buildCategory(items, name) {
      var _this5 = this;

      var categoryItems = items.map(function (item) {
        return preact.h(
          'div',
          { 'class': 'category-item flex-column', key: item.name },
          preact.h(
            'div',
            { 'class': 'item-title' },
            item.name
          ),
          preact.h(
            'div',
            { 'class': 'flex-centered-row' },
            preact.h(
              'div',
              { 'class': 'item-description' },
              item.description
            ),
            preact.h(
              'div',
              { 'class': 'consent-toggle' },
              preact.h(Toggle, {
                onChange: _this5.handleToggle.bind(_this5, item.name, item.origin),
                enabled: Boolean(_this5.state.consent.loadAll || _this5.state.consent && _this5.state.consent[item.origin] && _this5.state.consent[item.origin][item.name])
              })
            )
          )
        );
      });

      return preact.h(
        'div',
        { 'class': 'category', key: name },
        preact.h(
          'div',
          { 'class': 'category-title' },
          name
        ),
        preact.h(
          'div',
          { 'class': 'flex-centered-row' },
          preact.h(
            'div',
            { 'class': 'category-description' },
            this.state.categories[name]
          ),
          preact.h(
            'div',
            { 'class': 'consent-toggle' },
            !this.state.showCategories[name] && preact.h(Toggle, {
              onChange: this.handleToggle.bind(this, name, 'categories'),
              enabled: this.state.consent.loadAll || this.getCategoryToggle(name)
            })
          )
        ),
        preact.h(
          'div',
          { 'class': 'category-fold' },
          preact.h(
            'a',
            {
              onClick: function onClick() {
                _this5.handleFold(name);
              }
            },
            !this.state.showCategories[name] && 'See more',
            this.state.showCategories[name] && 'See less',
            preact.h(
              'svg',
              {
                width: '14',
                height: '8',
                xmlns: 'http://www.w3.org/2000/svg',
                'class': !this.state.showCategories[name]
              },
              preact.h('path', {
                d: 'M12.293.293L7 5.586 1.707.293A1 1 0 1 0 .293 1.707l6 6a.997.997 0 0 0 1.414 0l6-6A1 1 0 0 0 12.293.293',
                fill: '#1563FF',
                fillRule: 'evenodd'
              })
            )
          )
        ),
        this.state.showCategories[name] && preact.h(
          'div',
          { 'class': 'category-items flex-column' },
          categoryItems
        )
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _this6 = this;

      var categories = Object.keys(this.state.groupedIntegrations);
      categories = categories.map(function (category) {
        return _this6.buildCategory(_this6.state.groupedIntegrations[category], category);
      });

      return preact.h(
        'div',
        { 'class': 'g-consent-dialog' },
        !this.state.showConfirmationDialog && preact.h(
          'div',
          { 'class': 'g-consent-visible-dialog flex-column' },
          preact.h(
            'div',
            { 'class': 'dialog-title flex-centered-row' },
            preact.h(
              'span',
              null,
              'Data Collection Preferences'
            ),
            preact.h(
              'div',
              {
                'class': 'clickable flex-centered-row',
                onClick: function onClick() {
                  _this6.setState({ showConfirmationDialog: true });
                }
              },
              preact.h(CloseIcon, null)
            )
          ),
          preact.h(
            'div',
            { 'class': 'dialog-body' },
            preact.h(
              'p',
              null,
              this.props.companyName,
              ' uses data collected by cookies and JavaScript libraries to improve your browsing experience, analyze site traffic, and increase the overall performance of our site. By using our website, you\u2019re agreeing to our',
              ' ',
              preact.h(
                'a',
                {
                  href: this.props.privacyPolicyLink,
                  target: '_blank',
                  rel: 'noopener noreferrer'
                },
                'Privacy Policy'
              ),
              '.'
            ),
            preact.h(
              'p',
              null,
              'The categories below outline which companies and tools we use for collecting data. To opt out of a category of data collection, set the toggle to \u201COff\u201D and save your preferences.'
            ),
            preact.h(
              'div',
              { id: 'outline' },
              categories
            )
          ),
          preact.h(
            'div',
            { 'class': 'dialog-footer' },
            preact.h(Button, {
              title: 'Cancel',
              classes: 'button-cancel',
              theme: 'dark-outline',
              onClick: function onClick() {
                _this6.setState({ showConfirmationDialog: true });
              }
            }),
            preact.h(Button, {
              classes: 'button-save',
              title: 'Save Preferences',
              onClick: function onClick() {
                _this6.props.saveAndLoadAnalytics(_this6.state.consent);
              }
            })
          )
        ),
        this.state.showConfirmationDialog && preact.h(
          'div',
          { 'class': 'g-consent-visible-dialog' },
          preact.h(
            'div',
            { 'class': 'dialog-title flex-centered-row' },
            preact.h(
              'span',
              null,
              'Are you sure?'
            )
          ),
          preact.h(
            'div',
            { 'class': 'dialog-body' },
            preact.h(
              'p',
              null,
              'Your preferences have not been saved. To continue using our website, you must either set individual preferences or agree to',
              ' ',
              preact.h(
                'a',
                {
                  href: this.props.privacyPolicyLink,
                  target: '_blank',
                  rel: 'noopener noreferrer'
                },
                'our privacy policy'
              ),
              '.'
            )
          ),
          preact.h(
            'div',
            { 'class': 'dialog-footer' },
            preact.h(Button, {
              title: 'Back to Preferences',
              theme: 'dark-outline',
              classes: 'button-cancel',
              onClick: function onClick() {
                _this6.setState({ showConfirmationDialog: false });
              }
            }),
            preact.h(Button, {
              title: 'Agree & Close',
              classes: 'button-save',
              onClick: function onClick() {
                _this6.props.saveAndLoadAnalytics({ loadAll: true });
              }
            })
          )
        )
      );
    }
  }]);

  return ConsentPreferences;
}(preact.Component);

/*
  Cookies structure:
  {
    { segment: { Hotjar: true, Intercom: false, .. } },
    { custom: { OptinMonster: true } },
    { categories: { Functional: true } }
  }
*/
var COOKIE_KEY = 'hashi-consent-preferences';
var COOKIE_EXPIRES = 365;

function getDomain() {
  var host = window.location.hostname;
  var parts = host.split('.');
  var levels = [];
  var result = '';

  // get all domain levels
  for (var i = parts.length - 2; i >= 0; i--) {
    levels.push(parts.slice(i).join('.'));
  }

  // find top level domain
  for (var _i = 0; _i < levels.length; _i++) {
    var cname = '__tld__';
    var domain = levels[_i];
    var opts = { domain: '.' + domain };

    cookies.set(cname, 1, opts);
    if (cookies.get(cname)) {
      cookies.set(cname, null, opts);
      result = domain;
      break;
    }
  }

  return result;
}

function loadPreferences() {
  return cookies.getJSON(COOKIE_KEY);
}

function savePreferences(prefs, version) {
  var domain = getDomain();
  cookies.set(COOKIE_KEY, assign(prefs, { version: version }), {
    expires: COOKIE_EXPIRES,
    domain: domain
  });
}

/*
  load.js
  Responsible for loading scripts, whether they are Segment-originated or
  custom defined.
*/

function loadAnalytics(preferences, segmentWriteKey, additionalServices) {
  // If segment analytics is not on the page, warn and exit
  if (!window.analytics) {
    /* eslint-disable-next-line no-console */
    console.warn('Segment analytics not detected');
    return;
  }

  // If analytics have already been loaded, reload the page
  if (window.analytics.initialized) {
    window.location.reload();
    return;
  }

  if (preferences.loadAll) {
    loadSegment(segmentWriteKey, { All: true });
    loadCustom(additionalServices);
    return;
  }

  if (preferences.segment) {
    loadSegment(segmentWriteKey, preferences.segment);
  }

  if (preferences.custom) {
    loadCustom(additionalServices, preferences.custom);
  }
}

function loadSegment(writeKey, userPreferences) {
  window.analytics.load(writeKey, {
    integrations: assign({ All: false, 'Segment.io': true }, userPreferences)
  });
}

function loadCustom(services, userPreferences) {
  if (!services || services.length === 0) return;

  if (userPreferences) {
    // Load according to user preferences
    var integrationNames = Object.keys(userPreferences);
    var service = void 0;
    integrationNames.forEach(function (name) {
      service = services.find(function (service) {
        return userPreferences[name] && service.name === name;
      });
      if (service) {
        insertScript(service);
      }
    });
  } else {
    // Load everything
    services.forEach(function (service) {
      insertScript(service);
    });
  }
}

function insertScript(script) {
  var scriptElement = document.createElement('script');
  scriptElement.type = 'text/javascript';

  if (script.async) {
    scriptElement.setAttribute('async', '');
  }

  if (script.dataAttrs) {
    script.dataAttrs.forEach(function (attr) {
      scriptElement.dataset[attr.name] = attr.value;
    });
  }

  if (script.body) {
    scriptElement.text = script.body;
  } else if (script.url) {
    scriptElement.src = script.url;
  }

  if (script.addToBody) {
    document.body.appendChild(scriptElement);
  } else {
    document.head.appendChild(scriptElement);
  }
}

var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn$1(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits$1(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var emitter = new EventEmitter();

function init(props) {
  preact.render(preact.h(ConsentManager, props), document.querySelector(props.container));
}

function open() {
  emitter.emit('openDialog');
}

var ConsentManager = function (_Component) {
  _inherits$1(ConsentManager, _Component);

  function ConsentManager(props) {
    _classCallCheck$1(this, ConsentManager);

    var _this = _possibleConstructorReturn$1(this, (ConsentManager.__proto__ || Object.getPrototypeOf(ConsentManager)).call(this, props));

    _this.state = {
      showDialog: props.showDialog || false,
      showBanner: false,
      preferences: loadPreferences() || {}
    };
    _this.saveAndLoadAnalytics = _this.saveAndLoadAnalytics.bind(_this);
    _this.openDialog = _this.openDialog.bind(_this);
    return _this;
  }

  _createClass$1(ConsentManager, [{
    key: 'saveAndLoadAnalytics',
    value: function saveAndLoadAnalytics(preferences) {
      if (typeof preferences === 'undefined') {
        preferences = { All: false, 'Segment.io': false };
      }
      savePreferences(preferences, this.props.version);

      loadAnalytics(preferences, this.props.segmentWriteKey, this.state.additionalServices);

      // Close all dialogs
      document.body.classList.remove('g-noscroll');
      this.setState({
        showDialog: false,
        showBanner: false,
        preferences: loadPreferences()
      });
    }
  }, {
    key: 'shouldRequireConsent',
    value: function shouldRequireConsent() {
      // 1. Check cookies
      if (this.state.preferences && this.state.preferences.version === this.props.version) {
        // Load segment and custom integrations
        loadAnalytics(this.state.preferences, this.props.segmentWriteKey, this.state.additionalServices);
        return false;
      }
      // 2. Check inEU. If in-eu, show consent. If not, show nothing. Load all.
      if (inEU() || this.props.forceShow) {
        return true;
      } else {
        // Load all analytics
        this.saveAndLoadAnalytics({ loadAll: true });
        return false;
      }
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.setState({ showBanner: this.shouldRequireConsent() });
      emitter.on('openDialog', this.openDialog);
    }
  }, {
    key: 'openDialog',
    value: function openDialog() {
      document.body.classList.add('g-noscroll');
      this.setState({ showDialog: true, showBanner: false });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      return preact.h(
        'div',
        { id: 'g-consent-manager' },
        this.state.showBanner && preact.h(ConsentBanner, {
          privacyPolicyLink: this.props.privacyPolicyLink,
          onManagePreferences: this.openDialog,
          onAccept: function onAccept() {
            _this2.saveAndLoadAnalytics({ loadAll: true });
          }
        }),
        this.state.showDialog && preact.h(ConsentPreferences, {
          version: this.props.version,
          companyName: this.props.companyName,
          segmentWriteKey: this.props.segmentWriteKey,
          utilServerRoot: this.props.utilServerRoot,
          segmentServices: this.props.segmentServices,
          additionalServices: this.props.additionalServices,
          preferences: this.state.preferences,
          categories: this.props.categories,
          privacyPolicyLink: this.props.privacyPolicyLink,
          saveAndLoadAnalytics: this.saveAndLoadAnalytics
        })
      );
    }
  }]);

  return ConsentManager;
}(preact.Component);

exports.init = init;
exports.open = open;
exports.ConsentManager = ConsentManager;
